#version 330

uniform sampler2DRect inputTex0;
uniform sampler2DRect inputTex1;
uniform sampler2DRect inputTex2;
uniform sampler2DRect inputTex3;
uniform sampler2DRect inputTexMask;
uniform vec4 endColor;
uniform float endOpacity;


#ifdef AdobeRGB1998
    const vec4  profileToRGB_R = vec4(1.398283,-0.398283064, 0.0,0.0  );
    const vec4  profileToRGB_G = vec4(0.0,1.0,0.0,0.0  );
    const vec4  profileToRGB_B = vec4(0.0, -0.0429382771,1.04293835,0.0  );
#endif

#ifdef DISPLAY_P3
    const vec4  profileToRGB_R = vec4(1.2249,-0.2247, 0.0,0.0  );
    const vec4  profileToRGB_G = vec4(-0.0420,1.0419,0.0,0.0  );
    const vec4  profileToRGB_B = vec4(-0.027, -0.0786,1.0979,0.0  );
#endif

#ifdef BT2020
    // values obtained from:
    // Report ITU-R BT.2407-0 (10/2017) - Colour gamut conversion from Recommendation ITU-R BT.2020 to Recommendation ITU-R BT.709
    const vec4 profileToRGB_R = vec4( 1.6605, -0.5876, -0.0728, 0.0);
    const vec4 profileToRGB_G = vec4(-0.1246,  1.1329, -0.0083, 0.0);
    const vec4 profileToRGB_B = vec4(-0.0182, -0.1006,  1.1187, 0.0);
#endif

#ifdef FULL_RANGE
    //full range values from wikipedia https://en.wikipedia.org/wiki/YCbCr#ITU-R_BT.709_conversion
    const vec4 yuv2R = vec4( 1.402,1., 0.0,0.0  );
    const vec4 yuv2G = vec4(-0.714136,1.,-0.344136,0.0  );
    const vec4 yuv2B = vec4( 0.000000,1., 1.772,0.0  );
    const vec4 stdbias = vec4(-0.5  , 0.0,  -0.5   ,0.0  );
#else
    const vec4 yuv2R = vec4( 1.793,1.164383, 0.000000,0.0  );
    const vec4 yuv2G = vec4(-0.534,1.164383,-0.213,0.0  );
    const vec4 yuv2B = vec4( 0.000000,1.164383, 2.115,0.0  );
    const vec4 stdbias = vec4(-0.5  , -0.0625,  -0.5   ,0.0  );
#endif

const vec4 scale10to16 = vec4(64.0, 64.0, 64.0, 64.0);
const vec4 scale12to16 = vec4(16.0, 16.0, 16.0, 16.0);

vec4 toGamma(vec4 linearRGB)
{
    bvec3 cutoff = lessThan(linearRGB.rgb, vec3(0.0031308));
    vec3 higher = sqrt(linearRGB.rgb);
    vec3 lower = linearRGB.rgb * vec3(12.92);

    return vec4(mix(higher, lower, cutoff), linearRGB.a);
}

// Converts a color from sRGB gamma to linear light gamma
vec4 toLinear(vec4 sRGB)
{
    bvec3 cutoff = lessThan(sRGB.rgb, vec3(0.04045));
    vec3 higher = pow((sRGB.rgb + vec3(0.055))/vec3(1.055), vec3(2.4));
    vec3 lower = sRGB.rgb/vec3(12.92);

    return vec4(mix(higher, lower, cutoff), sRGB.a);
}

in vec2 fsTexY;
in vec2 fsTexUV;
in vec2 fsTexA;
in vec2 fsTextureMask;
out vec4 outColor;

void main()
{
    vec4 yuvIn;
    vec4 yuvOut;

    yuvIn.g = texture(inputTex0, fsTexY).r;
    yuvIn.b = texture(inputTex1, fsTexUV).r;
    yuvIn.r = texture(inputTex2, fsTexUV).r;
    
#ifdef HAS_ALPHA
    yuvIn.a = texture(inputTex3, fsTexA).r;
#endif

#ifdef IS_10BIT
    yuvIn*=scale10to16;
#endif
    
#ifdef IS_12BIT
    yuvIn*=scale12to16;
#endif
    
    yuvIn+=stdbias;
	
#ifndef HAS_ALPHA
    yuvIn.a=1.0;
#endif
    
    outColor=vec4(dot(yuv2R, yuvIn),
                  dot(yuv2G, yuvIn),
                  dot(yuv2B, yuvIn),
                  yuvIn.a);

#ifdef USE_COLOR_PROFILE
    vec4 linear=toLinear(outColor);

    vec4 srgbLinear=vec4(dot(profileToRGB_R, linear),
                    dot(profileToRGB_G, linear),
                    dot(profileToRGB_B, linear),
                    outColor.a);

    outColor=toGamma(srgbLinear);
#endif
    
#ifndef NO_PREMULT
    outColor.rgb*=outColor.a;
#endif

#ifdef USE_MASK
    outColor.a*=texture(inputTexMask,fsTextureMask).a;
#endif
    
#ifdef USE_END_COLOR
    outColor=mix(endColor,outColor,endOpacity);
#endif

}
